const ID_NO_PERSON = null;

export default {
    model: {
        prop: "personalNotes",
        event: "change",
    },
    created() {
        this.ID_NO_PERSON = ID_NO_PERSON;
    },
    methods: {
        removePersonalNote(personID) {
            if (personID === ID_NO_PERSON) {
                return
            }
            console.log("removing personal note of person", personID);
            this.personalNotes.forEach(item => {
                if (item.person.id === personID) {
                    item.tardiness = 0;
                    item.absent = false;
                    item.excused = false;
                    item.excuseType = null;
                    item.extraMarks = [];
                    item.remarks = "";
                }
            });
            this.$emit('change', this.personalNotes)
        },
        editPersonalNote(personID) {
            console.log("editing personal note of person", personID);
            this.editedPersonID = personID;
            this.updatePersonalNote();
            this.dialog = true;
        },
        updatePersonalNote() {
            let personalNote = this.personalNoteByStudentID(this.editedPersonID);
            this.editedTardiness = personalNote.tardiness || 0;
            this.editedAbsent = personalNote.absent || false;
            this.editedExcused = personalNote.excused || false;
            this.editedExcuseType = personalNote.excuseType || null;
            this.editedExtraMarks = personalNote.extraMarks || [];
            this.editedRemarks = personalNote.remarks || "";

            this.newPersonalNote = !!(personalNote && Object.keys(personalNote).length === 0 && Object.getPrototypeOf(personalNote) === Object.prototype);
        },
        createPersonalNote() {
            this.editedPersonID = ID_NO_PERSON;
            this.editedTardiness = 0;
            this.editedAbsent = false;
            this.editedExcused = false;
            this.editedExcuseType = null;
            this.editedExtraMarks = [];
            this.editedRemarks = "";
            this.newPersonalNote = true;
            this.dialog = true;
        },
        personalNoteByStudentID(studentID) {
            if (this.editedPersonID === ID_NO_PERSON) {
                return {};
            }
            return this.personalNotes.filter(item => item.person.id === studentID)[0] || {};
        },
        savePersonalNote() {
            if (this.editedPersonID === ID_NO_PERSON) {
                return
            }

            if (this.newPersonalNote) {
                this.personalNotes.push({
                    person: {
                        id: this.editedPersonID,
                        fullName: this.studentNameByID(this.editedPersonID)
                    },
                    tardiness: this.editedTardiness,
                    absent: this.editedAbsent,
                    excused: this.editedExcused,
                    excuseType: this.editedExcuseType,
                    extraMarks: this.editedExtraMarks,
                    remarks: this.editedRemarks,
                });
            } else {
                // Loop through all personal notes and update the ones that match the editedPersonID
                this.personalNotes.forEach(item => {
                    if (item.person.id === this.editedPersonID) {
                        item.tardiness = this.editedTardiness;
                        item.absent = this.editedAbsent;
                        item.excused = this.editedExcused;
                        item.excuseType = this.editedExcuseType;
                        item.extraMarks = this.editedExtraMarks;
                        item.remarks = this.editedRemarks;
                    }
                });
            }
            this.$emit('change', this.personalNotes)
        },
        cancelDialog() {
            this.dialog = false;
            this.editedPersonID = ID_NO_PERSON;
        },
        saveDialog() {
            this.savePersonalNote();
            this.dialog = false;
            this.editedPersonID = ID_NO_PERSON;
        },
        personalNoteString(personalNote) {
            let personalNoteString = "";
            if (personalNote.tardiness > 0) {
                personalNoteString += personalNote.tardiness + " min. ";
            }
            if (personalNote.absent) {
                personalNoteString += "abwesend ";
            }
            if (personalNote.excused) {
                personalNoteString += "entschuldigt ";
            }
            if (personalNote.excuseType) {
                personalNoteString += personalNote.excuseType.name;
            }
            if (personalNote.extraMarks.length > 0) {
                personalNoteString += " (";
                personalNote.extraMarks.forEach(item => {
                    personalNoteString += item.name + ", ";
                });
                personalNoteString = personalNoteString.substring(0, personalNoteString.length - 2);
                personalNoteString += ") ";
            }
            if (personalNote.remarks) {
                personalNoteString += "\"" + personalNote.remarks + "\" ";
            }
            return personalNoteString;
        },
        studentNameByID(studentID) {
            try {
                return this.persons.filter(item => item.id === studentID)[0].fullName;
            } catch (TypeError) {
                return "";
            }
        }
    },
    props: ["lessonDocumentationId", "personalNotes", "groups", "excuseTypes", "extraMarks"],
    name: "personal-notes",
    data: () => {
        return {
            dialog: false,
            // Absent versp. exc. type hw note
            editPersonalNoteId: null,
            editedPersonID: ID_NO_PERSON,
            editedTardiness: 0,
            editedAbsent: false,
            editedExcused: false,
            editedExcuseType: null,
            editedExtraMarks: [],
            editedRemarks: "",
            newPersonalNote: false,
        }
    },
    computed: {
        persons() {
            // go through each group and get the students
            // use the group names as headers for the v-select

            return this.groups.map(
                group => {
                    return [
                        {header: group.name, id: group.shortName},
                        group.members
                    ]
                }
            ).flat(2);
        }
    },
    template: `
    <v-dialog
      v-model="dialog"
      max-width="600px"
      @click:outside="cancelDialog"
    >
      <template v-slot:activator="{ on, attrs }">
        <div>
          <template v-for="personalNote in personalNotes">
            <v-chip class="ma-1" close @click="editPersonalNote(personalNote.person.id)"
              @click:close="removePersonalNote(personalNote.person.id)" v-if="personalNoteString(personalNote)">
              {{ personalNote.person.fullName }}: {{ personalNoteString(personalNote) }}
            </v-chip>
          </template>
        </div>
        <v-tooltip bottom>
          <template v-slot:activator="{ on, attrs }">
            <v-btn
              class="ma-1"
              color="primary"
              icon
              outlined
              v-bind="attrs"
              v-on="on"
              @click="createPersonalNote"
            >
              <v-icon>
                mdi-plus
              </v-icon>
            </v-btn>
          </template>
          <span v-text="$root.django.gettext('Add personal note')"></span>
        </v-tooltip>
      </template>
      <v-card>
        <v-card-title>
          <span class="text-h5">Personal Note</span>
        </v-card-title>
        <v-card-text>
          <v-container>
            <v-select
              item-text="fullName"
              item-value="id"
              :items="persons"
              label="Student"
              v-model="editedPersonID"
              @input="updatePersonalNote"
            ></v-select>
            <v-text-field
              label="Tardiness"
              suffix="min" type="number"
              min="0"
              :disabled="editedPersonID === ID_NO_PERSON"
              v-model="editedTardiness"
            ></v-text-field>
            <v-checkbox
              label="Absent"
              v-model="editedAbsent"
              :disabled="editedPersonID === ID_NO_PERSON"
              @change="editedExcused = false; editedExcuseType = null"
            ></v-checkbox>
            <v-checkbox
              label="Excused"
              v-model="editedExcused"
              :disabled="editedPersonID === ID_NO_PERSON || !editedAbsent"
              @change="editedExcuseType = null"
            ></v-checkbox>
            <v-select
              label="Excuse Type"
              v-model="editedExcuseType"
              :items="excuseTypes"
              item-text="name"
              item-value="shortName"
              :disabled="editedPersonID === ID_NO_PERSON || !editedAbsent || !editedExcused"
            ></v-select>
            <!-- FIXME: get extra mark itself as value -->
            <v-select
              label="Extra Marks"
              v-model="editedExtraMarks"
              :items="extraMarks"
              item-text="name"
              item-value="valueOf"
              :disabled="editedPersonID === ID_NO_PERSON"
              multiple
              chips
            ></v-select>
            <v-text-field
              label="Remarks"
              v-model="editedRemarks"
              :disabled="editedPersonID === ID_NO_PERSON"
            ></v-text-field>
          </v-container>
        </v-card-text>
        <v-card-actions>
          <v-spacer></v-spacer>
          <v-btn
            color="error"
            outlined
            @click="cancelDialog"
          >
            Cancel
          </v-btn>
          <v-btn
            color="success"
            @click="saveDialog"
            :disabled="editedPersonID === ID_NO_PERSON"
          >
            Save
          </v-btn>
        </v-card-actions>
      </v-card>
    </v-dialog>
    `
}